'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lr = require('@lezer/lr');
var highlight = require('@lezer/highlight');

// This file was generated by lezer-generator. You probably shouldn't edit it.
const Content = 1,
  StartTag = 2,
  StartAmpersandTag = 3,
  StartPartialTag = 4,
  StartSectionTag = 5,
  StartCloseSectionTag = 6,
  StartMismatchedCloseSectionTag = 7,
  MissingCloseSectionTag = 8,
  StartCommentTag = 9,
  StartUnescapedTag = 10,
  DelimiterTag = 11,
  InvalidDelimiterTag = 12,
  TagKey = 13,
  InvalidTagKey = 14,
  Comment = 15,
  EndTag = 16,
  EndUnescapedTag = 17,
  Element = 24,
  SectionTag = 25;

/* Hand-written tokenizer for mustache tag matching. */

function tagKeyChar(ch) {
  return (
    ch == 45 ||
    ch == 46 ||
    (ch >= 48 && ch <= 57) ||
    (ch >= 65 && ch <= 90) ||
    ch == 95 ||
    (ch >= 97 && ch <= 122)
  );
}

function delimiterChar(ch) {
  return ch != 32 && ch != 61 && ch > 0;
}

let cachedKey = null,
  cachedInput = null,
  cachedPos = 0;
function tagKeyAfter(input, offset, token) {
  let pos = input.pos + offset;
  if (cachedInput == input && cachedPos == pos) return cachedKey;
  while (input.peek(offset) == 32 /* ' ' */) offset++;
  let first = token.charCodeAt(0);
  let key = "";
  for (;;) {
    let next = input.peek(offset);
    if (!tagKeyChar(next)) break;
    if (next == first) {
      let len = 1;
      for (let l = token.length; len < l; len++)
        if (input.peek(offset + len) != token.charCodeAt(len)) break;
      if (len == token.length) break;
    }
    key += String.fromCharCode(next);
    offset++;
  }
  cachedInput = input;
  cachedPos = pos;
  return (cachedKey = key || null);
}

let cachedDelimiters = null,
  cachedDelimiterInput = null,
  cachedDelimiterPos = 0;
function delimitersAfter(input, offset) {
  let pos = input.pos + offset;
  if (cachedDelimiterInput == input && cachedDelimiterPos == pos)
    return cachedDelimiters;
  let delimiter = "";
  let startDelimiter = "";
  let endDelimiter = "";
  for (;;) {
    let next = input.peek(offset);
    if (!delimiterChar(next)) {
      if (next == 32 /* ' ' */ && startDelimiter == "") {
        startDelimiter = delimiter;
        delimiter = "";
        offset++;
        continue;
      } else if (next == 61 /* '=' */ && startDelimiter != "") {
        endDelimiter = delimiter;
        break;
      } else if (next < 0) {
        return;
      }
    }
    delimiter += String.fromCharCode(next);
    offset++;
  }
  if (startDelimiter == "" || endDelimiter == "") return null;
  cachedDelimiterInput = input;
  cachedDelimiterPos = pos;
  return (cachedDelimiters = [startDelimiter, endDelimiter] || null);
}

function Section(key, parent) {
  this.key = key;
  this.parent = parent;
}

function MustacheContext(section, delimiters) {
  this.section = section;
  this.delimiters = delimiters;
}

const mustacheContext = new lr.ContextTracker({
  start: new MustacheContext(null, ["{{", "}}"]),
  shift(context, term, stack, input) {
    return term == StartSectionTag
      ? new MustacheContext(
          new Section(
            tagKeyAfter(input, 3, context.delimiters[1]) || "",
            context.section,
          ),
          context.delimiters,
        )
      : term == DelimiterTag
        ? new MustacheContext(
            context.section,
            delimitersAfter(input, context.delimiters[1].length + 1) ??
              context.delimiters,
          )
        : context;
  },
  reduce(context, term) {
    return term == Element && context.section
      ? new MustacheContext(context.section?.parent, context.delimiters)
      : context;
  },
  reuse(context, node, _stack, input) {
    let type = node.type.id;
    return type == StartSectionTag || type == SectionTag
      ? new MustacheContext(
          new Section(
            tagKeyAfter(input, 3, context.delimiters[1]) || "",
            context.section,
          ),
          context.delimiters,
        )
      : context;
  },
  strict: false,
});

const content = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToToken(input, stack.context.delimiters[0])) {
      input.acceptToken(Content);
    }
  },
  { contextual: true },
);

const startTag = new lr.ExternalTokenizer(
  (input, stack) => {
    let startDelimiter = stack.context.delimiters[0];
    if (scanToken(input, startDelimiter)) {
      let next = input.next;
      if (next == 47 /* '/' */) {
        input.advance();
        let key = tagKeyAfter(input, 0, startDelimiter);
        if (!key) return input.acceptToken(StartCloseSectionTag);
        if (stack.context && key == stack.context.section?.key)
          return input.acceptToken(StartCloseSectionTag);
        for (let s = stack.context.section; s; s = s.parent)
          if (s.key == key) {
            return input.acceptToken(
              MissingCloseSectionTag,
              -1 - startDelimiter.length,
            );
          }
        input.acceptToken(StartMismatchedCloseSectionTag);
      } else if (next == 35 /* '#' */ || next == 94 /* '^' */) {
        input.advance();
        input.acceptToken(StartSectionTag);
      } else if (next == 33 /* '!' */) {
        input.advance();
        input.acceptToken(StartCommentTag);
      } else if (next == 123 /* '{' */) {
        input.advance();
        input.acceptToken(StartUnescapedTag);
      } else if (next == 61 /* '=' */) {
        input.advance();
        input.acceptToken(
          scanDelimiterTag(input, stack.context.delimiters[1])
            ? DelimiterTag
            : InvalidDelimiterTag,
        );
      } else if (next == 38 /* '&' */) {
        input.advance();
        input.acceptToken(StartAmpersandTag);
      } else if (next == 62 /* '>' */) {
        input.advance();
        input.acceptToken(StartPartialTag);
      } else input.acceptToken(StartTag);
    }
  },
  { contextual: true },
);

const tagKey = new lr.ExternalTokenizer(
  (input, stack) => {
    if (input.next == 32 /* ' ' */) return;
    let key = tagKeyAfter(input, 0, stack.context.delimiters[1]);
    let len = key?.length ?? 0;
    for (let i = 0; i < len; i++) input.advance();
    if (len > 1 && (key.startsWith(".") || key.indexOf("..") >= 0))
      input.acceptToken(InvalidTagKey);
    else if (len) input.acceptToken(TagKey);
  },
  { contextual: true },
);

const comment = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToToken(input, stack.context.delimiters[1])) {
      input.acceptToken(Comment);
    }
  },
  { contextual: true },
);

const endTag = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToken(input, stack.context.delimiters[1]))
      input.acceptToken(EndTag);
  },
  { contextual: true },
);
const endUnescapedTag = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToken(input, "}" + stack.context.delimiters[1]))
      input.acceptToken(EndUnescapedTag);
  },
  { contextual: true },
);

function scanToken(input, token) {
  for (let i = 0, l = token.length; i < l; i++) {
    if (input.peek(i) != token.charCodeAt(i)) return false;
  }
  for (let i = 0, l = token.length; i < l; i++) input.advance();
  return true;
}

function scanToToken(input, token) {
  let len = 0,
    first = token.charCodeAt(0);
  scan: for (; ; input.advance(), len++) {
    if (input.next < 0) break;
    if (input.next == first) {
      for (let i = 1; i < token.length; i++)
        if (input.peek(i) != token.charCodeAt(i)) continue scan;
      break;
    }
  }
  return len > 0;
}

function scanDelimiterTag(input, endToken) {
  if (input.next == 32 /* ' ' */) return false;
  while (delimiterChar(input.next)) input.advance();
  if (input.next == 32 /* ' ' */) input.advance();
  else return false;
  while (delimiterChar(input.next)) input.advance();
  if (input.next == 61 /* '=' */) input.advance();
  else return false;
  return scanToken(input, endToken);
}

const mustacheHighlighting = highlight.styleTags({
  "StartTag StartSectionTag StartCloseSectionTag MismatchedStartCloseSectionTag StartCommentTag StartPartialTag StartUnescapedTag StartAmpersandTag EndTag EndUnescapedTag DelimiterTag InvalidDelimiterTag":
    highlight.tags.strong,
  Content: highlight.tags.content,
  TagKey: highlight.tags.tagName,
  InvalidTagKey: [highlight.tags.tagName, highlight.tags.invalid],
  Comment: highlight.tags.blockComment,
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = lr.LRParser.deserialize({
  version: 14,
  states: "+|QQOROOOrO!fO'#C{OOOR'#Co'#CoO}O!fO'#C}OOOR'#Cp'#CpO!YOhO'#CqO!bO!fO'#DOOOOR'#Cr'#CrO!mO!fO'#CrO!xO!fO'#DPOOOR'#Cs'#CsO#TO!fO'#DQOOOR'#Cu'#CuO#`ORO'#CtOOOR'#Cz'#CzOOOR'#Cw'#CwQQOROOOOO!U'#Cx'#CxO#jO!fO,59gO#uO!rO,59gO#}O!fO,59iO$YO!rO,59iOOOR,59],59]O$bO`O,59]O$gO!fO,59jO$rO!rO,59jO$zO!fO,59^O%VO#SO,59^O%_O!fO,59kO%jO!rO,59kO%rO!fO,59lO%}O!rO,59lO&VO!fO'#DROOOR'#Cv'#CvOOOR,59`,59`O&bORO,59`OOOR-E6u-E6uOOO!U-E6v-E6vO&lO!rO1G/RO&lO!rO1G/ROOOR1G/R1G/RO&tO!rO1G/TO&tO!rO1G/TOOOR1G/T1G/TOOOR1G.w1G.wO&|O!rO1G/UO&|O!rO1G/UOOOR1G/U1G/UO'UO#SO1G.xO'UO#SO1G.xOOOR1G.x1G.xO'^O!rO1G/VO'^O!rO1G/VOOOR1G/V1G/VO'fO!rO1G/WO'fO!rO1G/WOOOR1G/W1G/WO'nO!fO,59mO'yO!rO,59mOOOR1G.z1G.zO(RO!rO7+$mOOOR7+$m7+$mO(ZO!rO7+$oOOOR7+$o7+$oO(cO!rO7+$pOOOR7+$p7+$pO(kO#SO7+$dOOOR7+$d7+$dO(sO!rO7+$qOOOR7+$q7+$qO({O!rO7+$rOOOR7+$r7+$rO)TO!rO1G/XO)TO!rO1G/XOOOR1G/X1G/XOOOR<<HX<<HXOOOR<<HZ<<HZOOOR<<H[<<H[OOOR<<HO<<HOOOOR<<H]<<H]OOOR<<H^<<H^O)]O!rO7+$sOOOR7+$s7+$sOOOR<<H_<<H_",
  stateData: ")e~OP^OQPORUOSROTZOVXOXTOYWOZ^O[^O~O]cO^cOpaO~O]eO^eOpaO~O_gO`fO~O]iO^iOpaO~O]kO^kOpaO~O]mO^mOpaO~O]oO^oOpaO~OUpOWrO~PQO]vO^vOpaO~O`xOpaO~O]yO^yOpaO~O`{OpaO~O`|O~O]}O^}OpaO~O`!POpaO~O]!QO^!QOpaO~Oa!SOpaO~O]!TO^!TOpaO~O`!VOpaO~O]!WO^!WOpaO~O`!YOpaO~O]![O^![OpaO~OUpOW!]O~PQO`!_OpaO~O`!aOpaO~O`!cOpaO~Oa!eOpaO~O`!gOpaO~O`!iOpaO~O]!jO^!jOpaO~O`!lOpaO~O`!mOpaO~O`!nOpaO~O`!oOpaO~Oa!pOpaO~O`!qOpaO~O`!rOpaO~O`!tOpaO~O`!uOpaO~O",
  goto: "%XvPPPPPPPPPPPPPPPPPPPwwwwww}!T!Z!eP$_$eP$k$q$w$}%TX^O]`sX]O]`sQr]R!]sQ`OQs]Tt`sQbPQdRQhUQjWQlXQnZzubdhjlnwz!O!R!U!X!Z!^!`!b!d!f!h!k!sQwcQzeQ!OiQ!RkQ!UmQ!XoQ!ZpQ!^vQ!`yQ!b}Q!d!QQ!f!TQ!h!WQ!k![R!s!jX_O]`sXQO]`sXSO]`sXVO]`sXYO]`sX[O]`sTq]s",
  nodeNames: "⚠ Content StartTag StartAmpersandTag StartPartialTag StartSectionTag StartCloseSectionTag StartMismatchedCloseSectionTag MissingCloseSectionTag StartCommentTag StartUnescapedTag DelimiterTag InvalidDelimiterTag TagKey InvalidTagKey Comment EndTag EndUnescapedTag Document Tag PartialTag CommentTag UnescapedTag MismatchedCloseSectionTag Element SectionTag CloseSectionTag",
  maxTerm: 37,
  context: mustacheContext,
  nodeProps: [
    ["closedBy", -7,2,3,4,5,6,7,9,"EndTag",10,"EndUnescapedTag",25,"CloseSectionTag"],
    ["openedBy", 16,"StartTag StartAmpersandTag StartPartialTag StartSectionTag StartCloseSectionTag StartMismatchedCloseSectionTag StartCommentTag",17,"StartUnescapedTag",26,"SectionTag"]
  ],
  propSources: [mustacheHighlighting],
  skippedNodes: [0],
  repeatNodeCount: 2,
  tokenData: "Z~RPpqU~ZOp~",
  tokenizers: [content, startTag, tagKey, comment, endTag, endUnescapedTag, 0],
  topRules: {"Document":[0,18]},
  tokenPrec: 0
});

exports.parser = parser;
